#!/bin/sh

sysctl vfs.devfs.generation > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -ne 0 ]; then
	if [ ! -f /dev/MAKEDEV.nvidia ]; then
		echo "ERROR: /dev/MAKEDEV.nvidia was not found!"
		exit 1
	fi
	cd /dev && ( sh /dev/MAKEDEV.nvidia all; cd .. )
fi

sysctl compat.linux.osname > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -ne 0 ]; then
	kldload linux > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -ne 0 ]; then
		echo "WARNING: Failed to load the Linux ABI module!"
		echo "WARNING: Is linux.ko installed?"
	fi
fi

sysctl compat.linux.osname > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	if [ -x /compat/linux/sbin/ldconfig ]; then
		/compat/linux/sbin/ldconfig > /dev/null 2>&1
	fi
fi

grep linux_enable /etc/rc.conf > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# Present.
	sed -e s/linux_enable.*/linux_enable=\"YES\"/g -i.orig /etc/rc.conf
else
	# Not present.
	echo 'linux_enable="YES"' >> /etc/rc.conf
fi

kldstat -v -n kernel | grep linux > /dev/null 2>&1 ; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# Linux ABI module built into kernel.
else
	grep linux_load /boot/loader.conf > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -eq 0 ]; then
		# Present.
		sed -e s/linux_load.*/linux_load=\"YES\"/g -i.orig /boot/loader.conf
	else
		# Not present.
		echo 'linux_load="YES"' >> /boot/loader.conf
	fi
fi

grep nvidia_load /boot/loader.conf > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# Present.
	sed -e s/nvidia_load.*/nvidia_load=\"YES\"/g -i.orig /boot/loader.conf
else
	# Not present.
	echo 'nvidia_load="YES"' >> /boot/loader.conf
fi

kldstat -n nvidia > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	kldunload -n nvidia > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -ne 0 ]; then
		echo 'ERROR: Failed to unload the NVIDIA module!'
		echo 'ERROR: Is nvidia.ko in use?'
		exit 1;
	fi
fi

kldload nvidia > /dev/null 2>&1 ; RESULT=$?
if [ ${RESULT} -ne 0 ]; then
	echo 'ERROR: Failed to load the NVIDIA module!'
	exit 1;
fi
